/*
** File: diagnostic.c  
** Project: ADMXRC3 module driver
** Purpose: OS-independent IOCTL handlers for diagnostic API functions.
**
** (C) Copyright Alpha Data 2010
*/

#include <df.h>
#include "device.h"
#include "diagnostic.h"
#include <admxrc3/types.h>

#if 0
void
sensorPollNotifyCallback(
  CoreNotification* pCoreNotification,
  unsigned int notifyType,
  void* pCallbackContext)
{
  FpgaInterruptNotification* pNotification = (FpgaInterruptNotification*)pCoreNotification;
  Admxrc3DeviceContext* pDevCtx = (Admxrc3DeviceContext*)pCallbackContext;

  dfDebugPrint(6, ("sensorPollNotifyCallback: pNotification=%p notifyType=%lu pCallbackContext=%p\n",
    pNotification, (unsigned long)notifyType, pCallbackContext));

  /* TO DO */
  pNotification = pNotification;
  pDevCtx = pDevCtx;
}
#endif

DfIoStatus
ioctlDiagGetSensorInfo(
	Admxrc3DeviceContext* pDevCtx,
  void* pBuffer,
  unsigned int inSize,
  unsigned int outSize)
{
  IOCTLS_ADMXRC3_GETSENSORINFO* pIoctl = (IOCTLS_ADMXRC3_GETSENSORINFO*)pBuffer;
  DfIoStatus status = DfIoStatusSuccess;
  uint32_t sensorIndex;
  CoreSensorStatus sensorStatus;
  CoreSensorInfo sensorInfo;

  if (NULL == pIoctl || sizeof(pIoctl->in) != inSize || sizeof(pIoctl->out) != outSize) {
    return DfIoStatusInvalid;
  }

  sensorIndex = pIoctl->in.sensorIndex;
  sensorStatus = pDevCtx->coreInterface.pGetSensorInfo(pDevCtx->pCoreContext, sensorIndex, &sensorInfo);
  if (CoreSensorSuccess != sensorStatus) {
    switch (sensorStatus) {
    case CoreSensorInvalidIndex:
      status = DfIoStatusError(ADMXRC3_INVALID_INDEX);
      break;

    default:
      status = DfIoStatusError(ADMXRC3_UNEXPECTED_ERROR);
      break;
    }
  } else {
    dfZeroMemory(&pIoctl->out, sizeof(pIoctl->out));
    dfStrCpy(pIoctl->out.description, DF_ARRAY_LENGTH(pIoctl->out.description), sensorInfo.description);
    pIoctl->out.capabilities = sensorInfo.capabilities;
    pIoctl->out.error.intPart = sensorInfo.error.intPart;
    pIoctl->out.error.fracPart = sensorInfo.error.fracPart;
    pIoctl->out.datatype = sensorInfo.datatype;
    pIoctl->out.unit = sensorInfo.unit;
    pIoctl->out.exponent = sensorInfo.exponent;
  }

  return status;
}

DfIoStatus
ioctlDiagReadSensor(
	Admxrc3DeviceContext* pDevCtx,
  void* pBuffer,
  unsigned int inSize,
  unsigned int outSize)
{
  IOCTLS_ADMXRC3_READSENSOR* pIoctl = (IOCTLS_ADMXRC3_READSENSOR*)pBuffer;
  DfIoStatus status = DfIoStatusSuccess;
  CoreSensorInfo sensorInfo;
  CoreSensorValue sensorValue;
  CoreSensorStatus sensorStatus;
  uint32_t sensorIndex;
  uint32_t datatype;

  if (NULL == pIoctl || sizeof(pIoctl->in) != inSize || sizeof(pIoctl->out) != outSize) {
    return DfIoStatusInvalid;
  }

  sensorIndex = pIoctl->in.sensorIndex;

  sensorStatus = pDevCtx->coreInterface.pGetSensorInfo(pDevCtx->pCoreContext, sensorIndex, &sensorInfo);
  if (CoreSensorSuccess != sensorStatus) {
    switch (sensorStatus) {
    case CoreSensorInvalidIndex:
      status = DfIoStatusError(ADMXRC3_INVALID_INDEX);
      break;

    case CoreSensorHardwareError:
    case CoreSensorUnexpectedError:
    default:
      status = DfIoStatusError(ADMXRC3_UNEXPECTED_ERROR);
      break;
    }
  }
  datatype = sensorInfo.datatype;

  sensorStatus = pDevCtx->coreInterface.pReadSensor(pDevCtx->pCoreContext, sensorIndex, &sensorValue);
  if (CoreSensorSuccess != sensorStatus) {
    switch (sensorStatus) {
    case CoreSensorInvalidIndex:
      status = DfIoStatusError(ADMXRC3_INVALID_INDEX);
      break;

    case CoreSensorHardwareError:
      status = DfIoStatusError(ADMXRC3_HARDWARE_ERROR);
      break;

    case CoreSensorUnexpectedError:
    default:
      status = DfIoStatusError(ADMXRC3_UNEXPECTED_ERROR);
      break;
    }
  }

  dfZeroMemory(&pIoctl->out, sizeof(pIoctl->out));
  switch (datatype) {
  case ADMXRC3_DATA_BOOL:
    pIoctl->out.value.booleanValue = sensorValue.booleanValue;
    break;

  case ADMXRC3_DATA_DOUBLE:
    pIoctl->out.value.doubleValue.intPart = sensorValue.doubleValue.intPart;
    pIoctl->out.value.doubleValue.fracPart = sensorValue.doubleValue.fracPart;
    break;

  case ADMXRC3_DATA_INT32:
    pIoctl->out.value.int32Value = sensorValue.int32Value;
    break;

  case ADMXRC3_DATA_UINT32:
    pIoctl->out.value.uint32Value = sensorValue.uint32Value;
    break;

  default:
    return DfIoStatusError(ADMXRC3_UNEXPECTED_ERROR);
  }
  pIoctl->out.datatype = datatype;

  return status;
}
